package cn.wonderbear.amap;

import android.graphics.BitmapFactory;
import android.location.Location;

import com.amap.api.maps.AMap;
import com.amap.api.maps.AMapOptions;
import com.amap.api.maps.CameraUpdateFactory;
import com.amap.api.maps.MapView;
import com.amap.api.maps.model.BitmapDescriptorFactory;
import com.amap.api.maps.model.LatLng;
import com.amap.api.maps.model.Marker;
import com.amap.api.maps.model.MarkerOptions;
import com.amap.api.maps.model.MyLocationStyle;
import com.amap.api.maps.model.Poi;
import com.amap.api.navi.AmapNaviPage;
import com.amap.api.navi.AmapNaviParams;
import com.amap.api.navi.AmapNaviType;
import com.facebook.infer.annotation.Assertions;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContext;
import com.facebook.react.bridge.ReadableArray;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.bridge.WritableMap;
import com.facebook.react.bridge.WritableNativeMap;
import com.facebook.react.common.MapBuilder;
import com.facebook.react.modules.core.RCTNativeAppEventEmitter;
import com.facebook.react.uimanager.SimpleViewManager;
import com.facebook.react.uimanager.ThemedReactContext;

import java.util.ArrayList;
import java.util.Map;

import javax.annotation.Nullable;

/**
 * Created by wonderbear on 14/07/2017.
 */

public class AMapManager extends SimpleViewManager<MapView> {

    private static final String REACT_CLASS = "AMap";
    private static final int COMMAND_SET_CENTER_COORDINATE = 1;
    private static final int COMMAND_RE_CENTER = 2;
    private static final int COMMAND_RELOAD_MAP = 3;
    private static final int COMMAND_ADD_ANNOTATION = 4;
    private static final int COMMAND_ADD_ANNOTATIONS = 5;
    private static final int COMMAND_SHOW_ANNOTATIONS = 6;
    private static final int COMMAND_CLEAR_ANNOTATIONS = 7;
    private static final int COMMAND_NAVI_TO = 10;
    private static final int COMMAND_NAVI_FROM_TO = 11;

    private boolean isRecenter = true;
    private MapView mapView;
    private final AMapOptions aMapOptions;
    private ReactApplicationContext reactApplicationContext;
    private Marker currentMarker = null;

    public AMapManager(ReactApplicationContext context) {
        this.reactApplicationContext = context;
        this.aMapOptions = new AMapOptions();
    }

    @Override
    public String getName() {
        return REACT_CLASS;
    }

    @Override
    public MapView createViewInstance(final ThemedReactContext context) {
        isRecenter = true;
        mapView = new MapView(context, aMapOptions);
        mapView.onCreate(null);
        AMap aMap = mapView.getMap();
        aMap.getUiSettings().setZoomControlsEnabled(false);// 设置缩放控件是否显示
        aMap.setMyLocationEnabled(true);// 设置为true表示启动显示定位蓝点，false表示隐藏定位蓝点并不进行定位，默认是false。
        MyLocationStyle myLocationStyle = new MyLocationStyle();
        myLocationStyle.myLocationType(MyLocationStyle.LOCATION_TYPE_LOCATION_ROTATE_NO_CENTER);
        aMap.setMyLocationStyle(myLocationStyle);
        aMap.setOnMapClickListener(new AMap.OnMapClickListener() {
            @Override
            public void onMapClick(LatLng latLng) {
                resizeMarker(currentMarker, false);
                sendEvent(context, "EventAnnotationDeselect", null);
            }
        });
        aMap.setOnMyLocationChangeListener(new AMap.OnMyLocationChangeListener() {
            @Override
            public void onMyLocationChange(Location location) {
                if (isRecenter) {
                    isRecenter = false;
                    AMap aMap = mapView.getMap();
                    aMap.animateCamera(CameraUpdateFactory.newLatLng(new LatLng(location.getLatitude(), location.getLongitude())));
                }
            }
        });
        aMap.setOnMarkerClickListener(new AMap.OnMarkerClickListener() {
            @Override
            public boolean onMarkerClick(Marker marker) {
                MarkerObject markerObject = (MarkerObject)(marker.getObject());
                if (markerObject != null) {
                    resizeMarker(currentMarker, false);
                    currentMarker = marker;
                    resizeMarker(currentMarker, true);
                }

                WritableNativeMap params = new WritableNativeMap();
                params.putDouble("latitude", marker.getPosition().latitude);
                params.putDouble("longitude", marker.getPosition().longitude);
                sendEvent(context, "EventAnnotationSelect", params);
                return true;
            }
        });
        return mapView;
    }

    @Nullable
    @Override
    public Map<String, Integer> getCommandsMap() {
        Map<String, Integer> map = MapBuilder.of();
        map.put("setCenterCoordinate", 1);
        map.put("reCenter", 2);
        map.put("reloadMap", 3);
        map.put("addAnnotation", 4);
        map.put("addAnnotations", 5);
        map.put("showAnnotations", 6);
        map.put("clearAnnotations", 7);
        map.put("naviTo", 10);
        map.put("naviFromTo", 11);
        return map;
    }

    @Override
    public void receiveCommand(MapView view, int commandId, @Nullable ReadableArray args) {
        Assertions.assertNotNull(view);
        Assertions.assertNotNull(args);
        switch (commandId) {
            case COMMAND_SET_CENTER_COORDINATE: {
                setCenterCoordinate(view, args);
                return;
            }
            case COMMAND_RE_CENTER: {
                reCenter(view);
                return;
            }
            case COMMAND_RELOAD_MAP: {
                reloadMap(view);
                return;
            }
            case COMMAND_ADD_ANNOTATION: {
                addAnnotation(view, args);
                return;
            }
            case COMMAND_ADD_ANNOTATIONS: {
                addAnnotations(view, args);
                return;
            }
            case COMMAND_SHOW_ANNOTATIONS: {
                showAnnotations(view, args);
                return;
            }
            case COMMAND_CLEAR_ANNOTATIONS: {
                clearAnnotations(view);
                return;
            }
            case COMMAND_NAVI_TO: {
                naviTo(view, args);
                return;
            }
            case COMMAND_NAVI_FROM_TO: {
                naviFromTo(view, args);
                return;
            }
            default:
                throw new IllegalArgumentException(String.format(
                        "Unsupported command %d received by %s.",
                        commandId,
                        getClass().getSimpleName()));
        }
    }

    private void sendEvent(ReactContext reactContext,
                           String eventName,
                           @Nullable WritableMap params) {
        reactContext
                .getJSModule(RCTNativeAppEventEmitter.class)
                .emit(eventName, params);
    }

    public void setCenterCoordinate(MapView view, @Nullable ReadableArray args) {
        if (args != null) {
            ReadableMap map = args.getMap(0);
            if (view != null && map.hasKey("longitude") && map.hasKey("latitude")) {
                AMap aMap = view.getMap();
                aMap.animateCamera(CameraUpdateFactory.newLatLng(new LatLng(map.getDouble("latitude"), map.getDouble("longitude"))));
            }
        }
    }

    public void reCenter(MapView view) {
        if (view != null) {
            AMap aMap = view.getMap();
            Location location = aMap.getMyLocation();
            if (location != null) {
                aMap.animateCamera(CameraUpdateFactory.newLatLng(new LatLng(location.getLatitude(), location.getLongitude())));
            }
        }
    }

    public void reloadMap(MapView view) {
        if (view != null) {
            AMap aMap = view.getMap();
            aMap.reloadMap();
        }
    }

    public void addAnnotation(MapView view, @Nullable ReadableArray args) {
        if (args != null) {
            ReadableMap map = args.getMap(0);
            MarkerOptions markerOptions = getMarkerOptions(map);
            if (view != null) {
                AMap aMap = view.getMap();
                Marker marker = aMap.addMarker(markerOptions);
                marker.setObject(getMarkerObject(map));
            }
        }
    }

    public void addAnnotations(MapView view, @Nullable ReadableArray args) {
        if (args != null) {
            ReadableArray array = args.getArray(0);
            ArrayList<MarkerOptions> markerOptions = new ArrayList<>();
            ArrayList<Marker> markers = new ArrayList<>();
            for (int i=0; i<array.size(); i++) {
                ReadableMap map = array.getMap(i);
                MarkerOptions markerOption = getMarkerOptions(map);
                markerOptions.add(markerOption);
            }
            if (view != null) {
                AMap aMap = view.getMap();
                markers = aMap.addMarkers(markerOptions, false);
            }
            for (int i=0; i<array.size(); i++) {
                ReadableMap map = array.getMap(i);
                MarkerObject markerObject = getMarkerObject(map);
                markers.get(i).setObject(markerObject);
            }
        }
    }

    public void showAnnotations(MapView view, @Nullable ReadableArray args) {
        clearAnnotations(view);
        addAnnotations(view, args);
    }

    public void clearAnnotations(MapView view) {
        if (view != null) {
            AMap aMap = view.getMap();
            aMap.clear();
        }
    }

    public void naviTo(MapView view, @Nullable ReadableArray args) {
        if (view != null && args != null) {
            AMap aMap = view.getMap();
            Location location = aMap.getMyLocation();
            if (location != null && args.size() > 0) {
                ReadableMap map = args.getMap(0);
                if (map.hasKey("location")) {
                    Poi start = new Poi("我的位置", new LatLng(location.getLatitude(), location.getLongitude()), "");
                    ReadableMap coord = map.getMap("location");
                    String title = map.hasKey("name") ? map.getString("name") : "";
                    Poi end = new Poi(title, new LatLng(coord.getDouble("latitude"), coord.getDouble("longitude")), "");
                    AmapNaviPage.getInstance().showRouteActivity(this.reactApplicationContext, new AmapNaviParams(start, null, end, AmapNaviType.DRIVER), null);
                }
            }
        }
    }

    public void naviFromTo(MapView view, @Nullable ReadableArray args) {
        if (view != null && args != null) {
            AMap aMap = view.getMap();
            Location location = aMap.getMyLocation();
            ReadableArray array = args.getArray(0);
            if (location != null && array.size() > 1) {
                ReadableMap mapSource = array.getMap(0);
                ReadableMap mapTarget = args.getMap(array.size() - 1);
                if (mapSource.hasKey("location") && mapTarget.hasKey("location")) {
                    ReadableMap coordSource = mapSource.getMap("location");
                    String titleSource = mapSource.hasKey("name") ? mapSource.getString("name") : "";
                    Poi start = new Poi(titleSource, new LatLng(coordSource.getDouble("latitude"), coordSource.getDouble("longitude")), "");
                    ReadableMap coordTarget = mapTarget.getMap("location");
                    String titleTarget = mapTarget.hasKey("name") ? mapTarget.getString("name") : "";
                    Poi end = new Poi(titleTarget, new LatLng(coordTarget.getDouble("latitude"), coordTarget.getDouble("longitude")), "");
                    AmapNaviPage.getInstance().showRouteActivity(this.reactApplicationContext, new AmapNaviParams(start, null, end, AmapNaviType.DRIVER), null);
                }
            }
        }
    }

    public MarkerOptions getMarkerOptions(ReadableMap map) {
        MarkerOptions markerOptions = new MarkerOptions();
        if (map.hasKey("coordinate")) {
            ReadableMap coord = map.getMap("coordinate");
            if (coord.hasKey("longitude") && coord.hasKey("latitude")) {
                markerOptions.position(new LatLng(coord.getDouble("latitude"), coord.getDouble("longitude")));
            }
        }
        markerOptions.title(map.hasKey("title") ? map.getString("title") : "");
        markerOptions.snippet(map.hasKey("snippet") ? map.getString("snippet") : "");
        String type = map.hasKey("type") ? map.getString("type") : "";
        switch (type) {
            case "car_care":
                markerOptions.icon(BitmapDescriptorFactory.fromBitmap(BitmapFactory
                        .decodeResource(reactApplicationContext.getResources(),R.drawable.location_car_care)));
                break;
            case "car_wash":
                markerOptions.icon(BitmapDescriptorFactory.fromBitmap(BitmapFactory
                        .decodeResource(reactApplicationContext.getResources(),R.drawable.location_car_wash)));
                break;
            case "car_mall":
                markerOptions.icon(BitmapDescriptorFactory.fromBitmap(BitmapFactory
                        .decodeResource(reactApplicationContext.getResources(),R.drawable.location_car_mall)));
                break;
            case "car_oil":
                markerOptions.icon(BitmapDescriptorFactory.fromBitmap(BitmapFactory
                        .decodeResource(reactApplicationContext.getResources(),R.drawable.location_car_oil)));
                break;
            case "car_park":
                markerOptions.icon(BitmapDescriptorFactory.fromBitmap(BitmapFactory
                        .decodeResource(reactApplicationContext.getResources(),R.drawable.location_car_park)));
                break;
            case "offence_high":
                markerOptions.icon(BitmapDescriptorFactory.fromBitmap(BitmapFactory
                        .decodeResource(reactApplicationContext.getResources(),R.drawable.location_offence_high)));
                break;
            case "offence_high_stop":
                markerOptions.icon(BitmapDescriptorFactory.fromBitmap(BitmapFactory
                        .decodeResource(reactApplicationContext.getResources(),R.drawable.location_offence_high_stop)));
                break;
            default:
                markerOptions.icon(BitmapDescriptorFactory.fromBitmap(BitmapFactory
                        .decodeResource(reactApplicationContext.getResources(),R.drawable.location_traffic)));
                break;
        }
        return markerOptions;
    }

    public MarkerObject getMarkerObject(ReadableMap map) {
        return new MarkerObject(map.hasKey("id") ? map.getInt("id") : 0, map.hasKey("type") ? map.getString("type") : "common");
    }

    public void resizeMarker(Marker marker, boolean isLarge) {
        if (marker != null) {
            if (isLarge) {
                marker.setIcon(BitmapDescriptorFactory.fromBitmap(BitmapFactory
                        .decodeResource(reactApplicationContext.getResources(),R.drawable.location_selected)));
            } else {
                MarkerObject markerObject = (MarkerObject)(marker.getObject());
                switch (markerObject.type) {
                    case "car_care":
                        marker.setIcon(BitmapDescriptorFactory.fromBitmap(BitmapFactory
                                .decodeResource(reactApplicationContext.getResources(),R.drawable.location_car_care)));
                        break;
                    case "car_wash":
                        marker.setIcon(BitmapDescriptorFactory.fromBitmap(BitmapFactory
                                .decodeResource(reactApplicationContext.getResources(),R.drawable.location_car_wash)));
                        break;
                    case "car_mall":
                        marker.setIcon(BitmapDescriptorFactory.fromBitmap(BitmapFactory
                                .decodeResource(reactApplicationContext.getResources(),R.drawable.location_car_mall)));
                        break;
                    case "car_oil":
                        marker.setIcon(BitmapDescriptorFactory.fromBitmap(BitmapFactory
                                .decodeResource(reactApplicationContext.getResources(),R.drawable.location_car_oil)));
                        break;
                    case "car_park":
                        marker.setIcon(BitmapDescriptorFactory.fromBitmap(BitmapFactory
                                .decodeResource(reactApplicationContext.getResources(),R.drawable.location_car_park)));
                        break;
                    case "offence_high":
                        marker.setIcon(BitmapDescriptorFactory.fromBitmap(BitmapFactory
                                .decodeResource(reactApplicationContext.getResources(),R.drawable.location_offence_high)));
                        break;
                    case "offence_high_stop":
                        marker.setIcon(BitmapDescriptorFactory.fromBitmap(BitmapFactory
                                .decodeResource(reactApplicationContext.getResources(),R.drawable.location_offence_high_stop)));
                        break;
                    default:
                        marker.setIcon(BitmapDescriptorFactory.fromBitmap(BitmapFactory
                                .decodeResource(reactApplicationContext.getResources(),R.drawable.location_traffic)));
                        break;
                }
            }
        }
    }
}
